'''
  exemple.iris.py

  Essai de fann. Traductoin en python de iris.c

  Voir la documentation fann/C++ : file:///usr/share/doc/libfann-dev/html/files/fann_cpp-h.html et se débrouiller !

  PP, 9/12/2024
'''

import fann2

# On crée l'objet réseau de neurones
ann = fann2.libfann.neural_net ()

num_input = 4
num_hidden = 3
num_output = 3
# On crée l'architecture du réseau de neurones
ann.create_standard_array ([num_input, num_hidden, num_output])
ann.set_activation_steepness_hidden (1)
ann.set_activation_steepness_output (1)
ann.set_activation_function_hidden (fann2.libfann.SIGMOID_SYMMETRIC)
ann.set_activation_function_output (fann2.libfann.LINEAR)
ann.set_train_stop_function (fann2.libfann.STOPFUNC_MSE)
ann.set_training_algorithm (fann2.libfann.TRAIN_RPROP)

# On charge le jeu de données de test
testdata = fann2.libfann.training_data ()
testdata.read_train_from_file ("iris.others")

import numpy as np
max_epochs = 10000
desired_error = 0
epochs_between_reports = 0

'''
  Pour chacun des fichiers de données d'entraînement iris.s[1-5], 
  on entraîne le réseau de neurones courant avec et on mesure le
  taux de succès sur le jeu de test.
'''
for batch in range (1,6):
    filename = "iris.s" + str (batch)
    traindata = fann2.libfann.training_data ()
    traindata.read_train_from_file (filename)
    ann.train_on_data (traindata, max_epochs, epochs_between_reports, desired_error)
    total_ok = 0
    for i in range (testdata.length_train_data()):
        calc_out = np.round (ann.run (testdata.get_input()[i]))
        total_ok += (np.sum (calc_out == testdata.get_output()[i]) == num_output)
    print ("Taux de succès : {}".format (total_ok / testdata.length_train_data()))
    traindata.destroy_train ()

# On sauvegarde le réseau
ann.save ("iris.py.net")

# Finalement, on détruit les objets.
testdata.destroy_train ()
ann.destroy ()
